pub mod fuse_overlay;
pub mod overlay;

use std::path::{Path, PathBuf};

pub trait Stack {
    fn lower_dirs(&self) -> impl Iterator<Item = &Path>;
    fn upper_dir(&self) -> Option<&Path>;
    fn work_dir(&self) -> Option<&Path>;
}

#[derive(Clone, Debug, PartialEq, Eq)]
pub struct Stackable {
    lower: Vec<PathBuf>,
    upper: Option<PathBuf>,
    work: Option<PathBuf>,
}

impl Stackable {
    pub fn new(lower: Vec<PathBuf>) -> Self {
        Self {
            lower,
            upper: None,
            work: None,
        }
    }

    pub fn with_upper_dir(&mut self, upper: impl Into<PathBuf>) -> &mut Self {
        self.upper = Some(upper.into());
        self
    }

    pub fn with_work_dir(&mut self, work: impl Into<PathBuf>) -> &mut Self {
        self.work = Some(work.into());
        self
    }
}

impl Stack for Stackable {
    fn lower_dirs(&self) -> impl Iterator<Item = &Path> {
        self.lower.iter().map(AsRef::as_ref)
    }

    fn upper_dir(&self) -> Option<&Path> {
        self.upper.as_deref()
    }

    fn work_dir(&self) -> Option<&Path> {
        self.work.as_deref()
    }
}
