use core::fmt::Debug;

pub trait MapWhileRefExt<'a, I: 'a> {
    fn map_while_ref<F>(&'a mut self, f: F) -> MapWhileRef<'a, I, F>;
}

impl<'a, I: Iterator + 'a> MapWhileRefExt<'a, I> for I {
    fn map_while_ref<F>(&mut self, f: F) -> MapWhileRef<I, F> {
        MapWhileRef { iter: self, f }
    }
}

#[derive(Debug)]
pub struct MapWhileRef<'a, I: 'a, F> {
    iter: &'a mut I,
    f: F,
}

impl<I, F, U, E> Iterator for MapWhileRef<'_, I, F>
where
    I: Iterator + Clone,
    F: FnMut(&I::Item) -> Result<U, E>,
{
    type Item = U;

    fn next(&mut self) -> Option<Self::Item> {
        let old = self.iter.clone();
        match self.iter.next() {
            None => None,
            Some(elt) => match (self.f)(&elt) {
                Ok(val) => Some(val),
                Err(_) => {
                    *self.iter = old;
                    None
                }
            },
        }
    }

    fn size_hint(&self) -> (usize, Option<usize>) {
        (0, self.iter.size_hint().1)
    }
}
